function [LP,Pinit,A,ar,sig2]=armurecu(Pinit,A,ar,sig2,Ts,b0,Oout,Oin)

// Baum-Welch for autoregressive HMMs training
// Only a single Gaussian observation sequence may be used.
// The formulas are based on Poritz algorithm. They are, however, extended to a single input-output model 
// (a multiple decorrelated outputs is also possible)
// This function takes the underflow problem into account 

// References:

// (case b0=0) A.B. Poritz. Linear predictive hidden Markov models and the speech signal. In Proc. ICASSP 82 (Paris, France),  pp. 1291-1294, May, 1982.
// (case b0=1) T. AL ANI. Extension of Poritz Linear Predictive Hidden Markov Models to Single Input Multi (decorrelated) Output Dynamic Systems.

// Ts   : number of observations at least represented by one state.
// Pinit   : initial state distribution (1*N).
// A    : state transition matrix (N*N)
// N    : number of states
// ar   : matrix of the regression coefficients in each state and for each
// output k (N*Ng)) where Ng=n+1 for b0=0 (AR model) or Ng=2(n+1) for b0=1 (ARX model). The ar(:,(k-1)*Ng+1), k=1,2,...K are always one.
// [N,K*(2(n+1)]=size(ar);		
//	     I 1 a1(1)     . . .  an(1) 1 b1(1)     . . .  bn(1)  I
// ar_k =    I .................................................  I
//           I 1 a1(N)     . . .  an(N) 1 b1(N)     . . .  bn(N)  I
// K    : Dimension of the system observation vector at time t.
// sig2	: Squared SIGMA matrix (K*N);
// values for sig2 have to be greater than zero
// b0	: b0=0 (u=0), AR model,  b0=1, ARX model
// Oout : output observation matrix of dimension (K*(T*Ts)), system output is a vector of size K.
// Oin  : input observation vector de dimension (1*(T*Ts)), system input is scalar.
// LP	: Normalized log of Likelihood of HMM (log(P(O|model)/T)

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

[K,T]=size(Oout);    // calculation of some variables from the function input
[N,N]=size(A);
[N,Np]=size(ar);
Ng=Np/K;
if b0==0 then
n=Ng-1; // number of regression coefficints for the input or output signal (n=na=nb).
elseif b0==1 then
n=Ng/2-1;
end
T=fix(T/Ts);
s=zeros(1,T);        // matrix with the scaling values
Alpha=zeros(N,T);    // Alpha : array to store the forward prob.
Beta=zeros(N,T);     // Beta :array to store the backward prob.
stop2=%f; 

Anew=A; arnew=ar; sig2new=sig2; // reestimated parameters

// calculation of Alpha for t=1

P=ones(N,1);

// calculation of the likelihood
for k=1:K 
    R=armake(Oout(k,1:Ts),Oin(1:Ts),n,b0);
    P=P.*arlike(ar(:,(k-1)*Ng+1:k*Ng),R,sig2(k,:));
end
Alpha(:,1)=Pinit'.*P;
s(1)=1/sum(Alpha(:,1));  
    if and(Alpha(:,1)==0),
	printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n");
	abort;
    end         
Alpha(:,1)=s(1)*Alpha(:,1);    // scaling 

// calculation of Alpha for t=2,...T

for t=2:T
    P=ones(N,1);
    for k=1:K                    // calculation of the likelihood
        R=armake(Oout(k,(t-1)*Ts+1:Ts*t),Oin((t-1)*Ts+1:Ts*t),n,b0);
        P=P.*arlike(ar(:,(k-1)*Ng+1:k*Ng),R,sig2(k,:));
    end
    Alpha(:,t)=(A'*(Alpha(:,t-1)).*P);
    if and(Alpha(:,t)==0),
	printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n");
	abort;
    end
    s(t)=1/sum(Alpha(:,t));
    Alpha(:,t)=s(t)*Alpha(:,t);   

end

// calculation of the likelihood that the given model has produced 
// the observation sequence

if or(s==0) then
  LP= -%inf
else
LP=-sum(log(s))/T; // normalized likelihood		
end

// calculation of Beta for t=T for i=1,..,N

Beta(:,T)=s(T)*ones(N,1);

// calculation of Beta for t=T-1,...,1 and for i=1,..,N 

for tk=1:T-1
    t=T-tk;
    P=ones(N,1);
    for k=1:K        // calculation of the likelihood
        R=armake(Oout(k,t*Ts+1:Ts*(t+1)),Oin(t*Ts+1:Ts*(t+1)),n,b0);
        P=P.*arlike(ar(:,(k-1)*Ng+1:k*Ng),R,sig2(k,:));
    end
    Beta(:,t)=s(t)*A*(P.*Beta(:,t+1));
end

// reestimation of the model parameters

// Reestimation of the State transition matrix (A)

SumDA=zeros(N,1);    // denominator to reestimate the transition matrix A
SumDsig=SumDA;       // denominator to reestimate the deviation matrix sig2

Is=ones(1,T)./s;
SumDsig=(Alpha.*Beta)*Is';
SumDA=SumDsig-Alpha(:,T).*Beta(:,T)/s(T);

if min(abs(SumDsig))==0,
	printf("Warning : Too small values in the denominator in the reestimation of C).\nThis leads to a division by zero.\n");
	printf("Program termination\n");
	abort;	
end
if min(abs(SumDA))==0,
	printf("Warning : Too small values in the denominator in the reestimation of A).\nThis leads to a division by zero.\n");
	printf("Program termination\n");
	abort;	
end

if or(SumDA<=0),            // sometimes problems with the precision
    LP=['error' 'error'];    // no valid solution possible 
else
    for i=1:N               // reestimation of a
        for j=1:N
            SumNAij=0;
            for t=1:T-1
                P=ones(N,1);
                for k=1:K
                    R=armake(Oout(k,t*Ts+1:Ts*(t+1)),Oin(t*Ts+1:Ts*(t+1)),n,b0);
                    P=P.*arlike(ar(:,(k-1)*Ng+1:k*Ng),R,sig2(k,:));
                end
                SumNAij=SumNAij+Alpha(i,t)*P(j)*Beta(j,t+1);    
            end
            Anew(i,j)=A(i,j)*SumNAij/SumDA(i);
        end
    end    
    
    for i=1:N             // reestimation of A and sig2
        for k=1:K
            Ri=0;
            for t=1:T
                R=armake(Oout(k,(t-1)*Ts+1:Ts*t),Oin((t-1)*Ts+1:Ts*t),n,b0);
                Ri=Ri+(Alpha(i,t)*Beta(i,t)/s(t))*R;
            end
            Di=Ri(2:Ng,2:Ng);
            Ci=Ri(1,2:Ng);
            if det(Di)==0,        // matrix not regular
                stop2=%t;
                break;
            else
                arn=-(Ci*inv(Di));
                arnew(i,(k-1)*Ng+1:k*Ng)=[1 arn];
                sig2new(k,i)=(ar(i,(k-1)*Ng+1:k*Ng)*Ri)*ar(i,(k-1)*Ng+1:k*Ng)'/SumDsig(i);
            end
        end
    end
    if and(sig2new>0),          // test of correctness
        A=Anew; ar=arnew; sig2=sig2new;
    end
end





