function [LP,Pinit,A,ar,sig2]=armurees(Pinit,A,ar,sig2,Ts,b0,Oout,Oin,tol,IterMax)

// This function recursively calls "armurecu" to train a ARHMM by Poritz algorithm.
// Only a single Gaussian observation sequence may be used.
// The formulas are based on Poritz algorithm. They are, however, extended to a single input-output model 
// (a multiple decorrelated outputs is also possible)

// References:

// (case b0=0) A.B. Poritz. Linear predictive hidden Markov models and the speech signal. In Proc. ICASSP 82 (Paris, France),  pp. 1291-1294, May, 1982.
// (case b0=1) T. AL ANI. Extension of Poritz Linear Predictive Hidden Markov Models to Single Input Multi (decorrelated) Output Dynamic Systems.

// Ts   : number of observations at least represented by one state.
// Pinit   : initial state distribution (1*N).
// A    : state transition matrix (N*N)
// N    : number of states
// ar   : matrix of the regression coefficients in each state and for each
// output k (Nx(K*Ng)) where Ng=n+1 for b0=0 (AR model) or Ng=2(n+1) for b0=1 (ARX model). The ar(:,(k-1)*Ng+1), k=1,2,...K are always one.
// [N,K*(2(n+1)]=size(ar);		
//	     I 1 a1(1)     . . .  an(1) 1 b1(1)     . . .  bn(1)  I
// ar_k =    I .................................................  I
//           I 1 a1(N)     . . .  an(N) 1 b1(N)     . . .  bn(N)  I
// K    : Dimension of the system observation vector at time t.
// sig2	: Squared SIGMA matrix (K,N);
// values for sig2 have to be greater than zero
// b0	: b0=0 (u=0), AR model,  b0=1, ARX model
// Oout : output observation matrix of dimension (K*(T*Ts)), system output is a vector of size K.
// Oin  : input observation vector de dimension (1*(T*Ts)), system input is scalar.
// tol 	: Stop if abs((New log(likelihood) - old log(liklihood)) / old log(likelihood)) is less than tol or Iteration > IterMax. 
// IterMax : Maximum Iteration number
// LP	: Normalized log of Likelihood of HMM (log(P(O|model)/T)

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=10,
	printf("Wrong number of input arguments in function ARMUREES. Function call is:\n");
	printf("[ LP , Pinit , A , ar , sig2 ] = armurees ( Pinit, A , ar , sig2 , Ts , b0 , Oout , Oin , tol , IterMax)");
	printf("Program termination\n");
	abort;
end
if lhs~=5,// check if number of output arguments is correct
	printf("Wrong number of output arguments. Continue ? ( y/n )\n");
	printf("If you want to continue type  y  else  n : \n");
	t=read(%io(1),1,1,'(a)');
	if (t~='y')&(t~='Y')
		printf("Program termination\n");
		abort;
	end
end
printf("====> Requested tolerance tol (to be compared to RCNL) = %5.10f\n",tol);
printf("====> Requested Maximum iteration IterMax (if tol is not satisfied) = %d\n",IterMax);
printf("**> Iteration = 1\n");
[LProb,Pinit,A,ar,sig2]=armurecu(Pinit,A,ar,sig2,Ts,b0,Oout,Oin);
printf("-> Normalized Log of Likelihood (log(P(O|model))/T) = %5.10f\n",LProb);
LProbold=LProb;
LP=LProb;
printf("**> Iteration = 2\n");
[LProb,Pinit,A,ar,sig2]=armurecu(Pinit,A,ar,sig2,Ts,b0,Oout,Oin);
printf("-> Normalized Log of Likelihood (log(P(O))/T) = %5.10f\n",LProb);
RCNL=abs((LProb-LProbold)/LProbold);
printf(">> Relative change in Normalized Log of Likelihood (RCNL=abs((New log(likelihood) - old log(liklihood)) / old log(likelihood))) = %5.10f\n",RCNL);	
Iter=2;
LP=[LP LProb];
while ((RCNL>tol)&(Iter<IterMax))
	LProbold=LProb
	Iter=Iter+1;
	printf("**> Iteration = %d\n",Iter);
	[LProb,Pinit,A,ar,sig2]=armurecu(Pinit,A,ar,sig2,Ts,b0,Oout,Oin);
	printf("-> Normalized Log of Likelihood (log(P(O|model))/T) = %5.10f\n",LProb);
	RCNL=abs((LProb-LProbold)/LProbold);
	printf(">> Relative change in Normalized Log of Likelihood (RCNL=abs((New log(likelihood) - old log(liklihood)) / old log(likelihood))) = %5.10f\n",RCNL);	
   	LP=[LP LProb];
end
