function [LP,A,Me,Seg]=cannlr(a,N,O,StartSeq,Iter)

// CHMM training by simulated annealing approach

// function used for training a general left-to-right CHMM
// by simulated annealing for a single gaussian (M=1).
// The matrix 'a' indicates the allowed transitions (1 for allowed, 0 for forbidden transitions),
// necessary in left-to right models.
// Since the algorithm uses 'a' only as a penalty matrix, it is not guarenteed, that the result
// has the desired form. A longer iteration often solves this problem. At the end of the
// simulated annealing procedure it might, however, be necessary to swap lines in the A-matrix
// and the corresponding lines in the other matrices to obtain the desired result.
// If 'a' is given as ones(N,N), it is assumed that all transitions are allowed.
// The variable Iter gives the number of iterations with different ( reducing )
// temperatures
// Brief description of the algorithm : Given the observation sequence O, a state sequence is
// generated at random. With the observations that are then assigned to each state, the mean and the
// variance of the sample are calculated.
// With these values the probability of generating O with the guessed state sequence is calculated.
// This is called the inital cost.
// Afterwards one state in the sequence is changed at random and the probability
// that this new state sequence has generated O is calculated, after changing the mean and the variance
// of the changed states.
// If the new probability is bigger than the old one, or the probability of acceptance
// Paccept=exp((Pnew-Pold)/Temp) is big enough, the change is saved. The probability of acceptance
// decreases with decreasing temperature so that for longer iteration only better solutions are accepted.

// a            :(N*N) Matrix that indicate the allowed transitions (1 for allowed, 0 for forbidden transitions),
// necessary in left-to right models.
// LP       : Normalized log of likelihood of HMM (log(P(O|model)/T)
// P        : Likelihood
// A        : State stochastic matrix (N*N)
// Seg      : Observation Covariance list Seg=list(1:M)Segi
// Segi     : Observation covarianc list Segi=list(1:N)Segsi
// Segsi    : Covariance Matrix (K*K) in state si
// Me       : Observation mean list Me=list(1:M)Mi
// Mi       : Matrices  Mi (K*N)of mean vectors Msi
// Msi      : Observation Mean vectors (K*1) in state si
// C        : Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*1)  : Case of Single Gaussian
// C (N*M)  : Case of M mixtures
// Oi       : Single Gaussian Observation matrix (K*T)
// O        : Concatenated observation sequence matrices (K*nT)
// X        : State sequence
// M        : Number of Gaussian observation mixtures (scalar)
// K        : Observation mean vector dimension
// N        : Number of states in HMM (scalar)
// T        : Concatenated Observation sequence length (scalar). Example if O=[O1 O2]
//            with size(O1,2)=T1 and size(O2,2)=T2, then T=T1+T2.
// StartSeq : A line vector indicating the start index in O of the ieme concatenated observation sequence.
//            Example : [1 201]' , 1 for sequence 1 of length=200 and 201 for sequence 2 of length 200.
// Iter     : Number of iterations with different ( reducing ) temperatures
// Pinit    : Initial state distribution vector (1*N), Pinit=[1 0 0 ...]

//AUTHORS:

// Copyright (c) 2000 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

//check of the function call

[lhs,rhs]=argn(0);
if rhs~=5;
    printf('Wrong number of input arguments in function CANNEALS. Function call is:\n ');
    printf('[ LP, A , Me , Seg ] = canneals ( a , N , O ,StartSeq, Iter, Rec )');
    printf('Program termination\n');
    abort;
end

if lhs~=4
    printf('Wrong number of output variables\n');
    printf('Continue? (y/n)\n');
    ans=read(%io(1),1,1,'(a)');
    if ~((ans=='y')|(ans=='Y')),
        printf('Program termination\n');
        abort;
    end
end

ExecTime=timer();
printf('>> Simulated Annealing running for %d Temperature levels\n',Iter);
printf('   Please be patient we will keep you informed of the remaining Time\n')

[K,T]=size(O);                  // T : observation sequence length K : number of observations per state
A=zeros(N,N)+sqrt(%eps);    // A : estimated state stochastic matrix
Asum=A;                     // Asum : used to add the number of occurences of a state
C=ones(K,1);                // C : known gain matrix
//Temp=1;                   // Temp : temperature
X=zeros(T,1);               // X : guess of the state sequence
Chain=T*3;                  // length of Iteration with fixed temperature
Alpha=0.9;                  // reduction factor for the temperature
Beta=1.1;                   // increasing factor for the iteration length
acceptance = 0.95;              // Initial acceptance ratio
c=(2*%pi)^(-K/2);
logc=log(c);


rand('uniform');                // sets random number generator to uniform

VarTot=list();
for i=1:T
  VarTot(i)=O(:,i)*O(:,i)';
end  //end of for

// Initialisation

if or((a~=1 & a~=0)),       // check of the matrix a
  printf('The matrix ''a'' must consist of ones and zeros ! Programm termination\n');
  abort;
end //end if

if and(size(a)==1),     // check if 'a' is a scalar
    a=ones(N,N);        // all transitions are allowed
end //end if

S=ones(N,1);
a=[a,-S];
Free=[];
for k=1:T
  Free=[Free;%t];
end
for t=1:size(StartSeq,1)
  Free(StartSeq(t))=%f;
  X(t)=1;
end

for k=1:size(StartSeq,1)
   X(StartSeq(k))=1;                 // always assumed Pinit=[1 0 0 ...]
end;
for t=2:T               // guess of the state sequence
    if Free(t)
      X(t)=fix(rand(1)*N)+1;
    end
    Asum(X(t-1),X(t))= Asum(X(t-1),X(t))+1;
end  //end of for
Asum=[Asum,Asum*S];

SumSq = list();
Seg = list();
SumObs = [];
Me = [];
NumObs=zeros(N,1);

for j=1:N           // assigning the observations to the states and calculating A, Me, Seg
    v=0;                // number of observations per state
        SSq=zeros(K,K);
        SObs=zeros(K,1);
    for t=1:T
        if X(t)==j,             // assignment of the observations according to the state sequence
            NumObs(j)=NumObs(j)+1;
            SSq=SSq+VarTot(t);
                        SObs=SObs+O(:,t);
        end //end if
    end  //end of for t

// calculating mean and variance
        SumSq(j) = SSq;
        SumObs = [SumObs,SObs];
        meanO = SObs/NumObs(j);
        varianceO = SSq/(NumObs(j)-1)                    // covariance matrix of the sample
        Me = [Me,meanO];
    Seg(j) = varianceO - (NumObs(j)/(NumObs(j)-1))*meanO*meanO';
end  //end of for j

for i=1:N
  Segi = Seg(i);
  SegInv = inv(Segi);
  Pb(i)= - 0.5*log(det(Segi))*NumObs(i) -0.5* sum(SumSq(i).*SegInv) + 0.5*NumObs(i)*Me(:,i)'*SegInv*Me(:,i);
end //end of for

Prob=sum(a.*Asum.*log(Asum))+sum(Pb);

BestMe=Me;                                            // save of the best guessed variables so far
BestSeg=Seg;
BestA=A;
BestP=Prob;

Count = 0;
Delta = 0;

// Calculation of the initial temperature

for trial=1:Chain
  NotFound=%t;
  while NotFound
    time=fix(rand(1)*(T-1))+2;        // Choice of period for changing X
    if Free(time), NotFound=%f; end
  end
  stop=%f;

  while ~stop,              // Choice of state to change to
   Xnew=fix(rand(1)*N)+1;
   if Xnew ~= X(time), stop=%t; end //end of if
  end //end of while
  Asumest=Asum;     // save the old estimates
  Segest=Seg;
  Meest=Me;
  SumSqest=SumSq;
  SumObsest=SumObs;
  NumObsest=NumObs;
  Pbest=Pb;
  Asum(X(time-1),X(time))=Asum(X(time-1),X(time))-1;//change of Asum according to the new state
  Asum(X(time-1),Xnew)=Asum(X(time-1),Xnew)+1;      // sequence

  if time < T,
   Asum(X(time),X(time+1))=Asum(X(time),X(time+1))-1;
   Asum(Xnew,X(time+1))=Asum(Xnew,X(time+1))+1;
  end // end of if

// Changes for the old state and the new state
  Xold=X(time);
  X(time)=Xnew;
  Asum(:,N+1)=sum(Asum(:,1:N),2);


  SSq=SumSq(Xold)-VarTot(time);
  SumSq(Xold)=SSq;
  SumObs(:,Xold)=SumObs(:,Xold)-O(:,time);
  NumObs(Xold)=NumObs(Xold)-1;
  SSq=SSq/(NumObs(Xold)-1);             // covariance matrix of the sample
  meanO=SumObs(:,Xold)/NumObs(Xold);
  Me(:,Xold)= meanO;
  Seg(Xold) = SSq - (NumObs(Xold)/(NumObs(Xold)-1))*meanO*meanO';
  SSq=SumSq(Xnew)+VarTot(time);
  SumSq(Xnew)=SSq;
  SumObs(:,Xnew)=SumObs(:,Xnew)+O(:,time);
  NumObs(Xnew)=NumObs(Xnew)+1;
  SSq=SSq/(NumObs(Xnew)-1);             // covariance matrix of the sample
  meanO=SumObs(:,Xnew)/NumObs(Xnew);
  Me(:,Xnew)=meanO;
  Seg(Xnew) = SSq - (NumObs(Xnew)/(NumObs(Xnew)-1))*meanO*meanO';
  SegInv = inv(Seg(Xold));
  Pb(Xold)=-0.5*log(det(Seg(Xold)))*NumObs(Xold)-0.5*sum(SumSq(Xold).*SegInv)...
           +0.5*NumObs(Xold)*Me(:,Xold)'*SegInv*Me(:,Xold);
  SegInv = inv(Seg(Xnew));
  Pb(Xnew)=-0.5*log(det(Seg(Xnew)))*NumObs(Xnew)-0.5*sum(SumSq(Xnew).*SegInv)...
           +0.5*NumObs(Xnew)*Me(:,Xnew)'*SegInv*Me(:,Xnew);

  Pest=sum(a.*Asum.*log(Asum))+sum(Pb);

  if (Pest>Prob),
    Count=Count+1;
    Delta=Delta+(Pest-Prob);
  end // end if

  Prob=Pest;

end //end for trial

Delta=Delta/Count;
Al = Count/Chain;
Temp=Delta/log((1-Al)/(acceptance-Al));


// Start simulated annealing

continue = %t;
ExecTime=timer();
Chain = T;
for k=1:Iter
        if ~continue then break; end //end if
    Chain=fix(Chain*Beta);
    Temp=Alpha*Temp;
        Accepted = 0;
    for trial=1:Chain
        NotFound=%t;
        while NotFound
          time=fix(rand(1)*(T-1))+2;        // Choice of period for changing X
          if Free(time), NotFound=%f; end
        end

         stop=%f;

        while ~stop,                // Choice of state to change to
            Xnew=fix(rand(1)*N)+1;

            if Xnew ~= X(time),
                stop=%t;
            end //end of if

        end //end of while
        Asumest=Asum;       // save the old estimates
        Segest=Seg;
        Meest=Me;
                SumSqest=SumSq;
                SumObsest=SumObs;
                NumObsest=NumObs;
                Pbest = Pb;

        Asumest(X(time-1),X(time)) = Asumest(X(time-1),X(time))-1;//change of Asum according to the
        Asumest(X(time-1),Xnew) = Asumest(X(time-1),Xnew)+1;      //new state sequence

        if time < T,
            Asumest(X(time),X(time+1))=Asumest(X(time),X(time+1))-1;
            Asumest(Xnew,X(time+1))=Asumest(Xnew,X(time+1))+1;
        end // end of if

// Changes for the old state and the new state
        Xold=X(time);
        X(time)=Xnew;
        Asumest(:,N+1)=sum(Asumest(:,1:N),2);

                SumSqest(Xold)=SumSqest(Xold)-VarTot(time);
                SumObsest(:,Xold)=SumObsest(:,Xold)-O(:,time);
                NumObsest(Xold)=NumObsest(Xold)-1;
                Meest(:,Xold)=SumObsest(:,Xold)/NumObsest(Xold);
        Segest(Xold)=(SumSqest(Xold)- NumObsest(Xold)* Meest(:,Xold)*Meest(:,Xold)')/(NumObsest(Xold)-1);
                SumSqest(Xnew)=SumSqest(Xnew)+VarTot(time);
                SumObsest(:,Xnew)=SumObsest(:,Xnew)+O(:,time);
                NumObsest(Xnew)=NumObsest(Xnew)+1;
                Meest(:,Xnew)=SumObsest(:,Xnew)/NumObsest(Xnew);
        Segest(Xnew)=(SumSqest(Xnew)- NumObsest(Xnew)* Meest(:,Xnew)*Meest(:,Xnew)')/(NumObsest(Xnew)-1);
                SegInv = inv(Segest(Xold));
                Pbest(Xold)=-0.5*log(det(Segest(Xold)))*NumObsest(Xold)-0.5*sum(SumSqest(Xold).*SegInv)...
                            +0.5*NumObsest(Xold)*Meest(:,Xold)'*SegInv*Meest(:,Xold);
                SegInv = inv(Segest(Xnew));
                Pbest(Xnew)=-0.5*log(det(Segest(Xnew)))*NumObsest(Xnew)-0.5*sum(SumSqest(Xnew).*SegInv)...
                            +0.5*NumObsest(Xnew)*Meest(:,Xnew)'*SegInv*Meest(:,Xnew);

                Pest=sum(a.*Asumest.*log(Asumest))+sum(Pbest);

            // acceptance of the new state sequence if the new probability is greater then the old one
        // or the probability of acceptance Paccept=exp((Pest-Prob)/Temp) is big enough

        if (Pest>Prob | exp((Pest-Prob)/Temp) > rand(1)),
            Prob=Pest;
                        Accepted = Accepted + 1;
            Asum=Asumest;
            Seg=Segest;
            Me=Meest;
                        SumSq=SumSqest;
                        SumObs=SumObsest;
                        NumObs=NumObsest;
                        Pb=Pbest;

            if Pest>BestP,      // change of the best estimated variables so far
                BestMe=Me;
                BestSeg=Seg;
                BestP=Pest;
                                BestAsum=Asum;
            end //end if
                else X(time)=Xold;
        end //end if

      end //end for trial
      Tk_1= timer();
      nrem = Iter-k;
      Trem = Tk_1*(Beta*(Beta^nrem - 1)/(Beta - 1));
      select %t
       case (Trem < 300)  then
        printf('** Execution time at Temperature level %d is  %d s. Estimated remaining Time : %d s\n',...
               k,int(Tk_1),int(Trem));
       case (Trem >= 300 & Trem < 3600)  then
        printf('** Execution time at Temperature level %d is  %d s. Estimated remaining Time : %d mn\n',...
               k,int(Tk_1),int(Trem/60));
       else
        printf('** Execution time at Temperature level %d is  %d s. Estimated remaining Time : %d h %d mn\n',...
               k,int(Tk_1),int(Trem/3600),int((Trem-int(Trem/3600)*3600)/60));
      end  // end of select
      Accepted = Accepted * 100 / Chain;
      printf('>> Acceptance Rate = %6.2f  Current Normalized Log of Probability = %e\n',Accepted,BestP/T);
end //end for k

Me=BestMe; Seg=BestSeg;
BestAsum=BestAsum(:,1:N);
A=BestAsum./(sum(BestAsum,2)*ones(1,N));

// The variable Vit gives the number of iterations of an optimisation using the
// Viterbi algorithm following the simulated annealing procedure.

BestMe=Me;
BestSeg=Seg;
Me=list();
Seg=list();
Me(1)=BestMe;
LP=Prob;
Seg(1)=BestSeg;
//Vit=1;
//tol=0.000001;
//printf('>> Five iteration for further optimisation using the Viterbi algorithm\n');
//[LP,A,Me,Seg]=cvireest(Pinit,A,Me,Seg,O,tol,IterMax);
