function [ok] = cchkparm(Pinit,A,Me,Seg,C,O)

// This function checks if the matrices Pinit,A, C and the lists Me,Seg are valid
// for a Multivariate Mixture Gaussian Observations HMM, i.e. checks if the sizes match, the sum of 
// probabilities is 1, probabilities are > 0 and covariance martices
// are symmetric and positiv definit, mixtures gain factors are > 0 and sum to 1
 
// Pinit 	: Initial state distribution vector (1*N)
// A     	: State stochastic matrix (N*N)
// Seg   	: Observation Covariance list Seg=list(1:M)Segi
// Segi 	: Observation covarianc list Segi=list(1:N)Segsi
// Segsi	: Covariance Matrix (K*K) in state si 
// Me    	: Observation mean list Me=list(1:M)Mi
// Mi		: Matrices  Mi (K*N)of mean vectors Msi
// Msi		: Observation Mean vectors (K*1) in state i 
// C     	: Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*1) 	: Case of Single Gaussian
// C (N*M)	: Case of M mixtures
// M		: Number of Gaussian observation mixtures (scalar)
// K		: Observation mean vector dimension
// N		: Number of states in HMM (scalar)
// T     	: Observation sequence length (scalar)
// O 		: Observation sequence matrix (K*T) 
// ok		: A logical variable (T or F) 

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved


[lhs,rhs]=argn(0);

ok=%t;				// value to be returned if everything is correct

[rPinit,cPinit]=size(Pinit);
[N,cA]=size(A);
K=size(O,1);
M=size(Seg);
[rC,cC]=size(C);

// check of Pinit

flag=%t;
if (rPinit~=1),	// ckeck of size of Pinit
	printf("Pinit is not a vector\n");
 	ok=%f; 
end
for i=1:N
	if Pinit(i)<0,	// check if all elements of Pinit are > 0
		printf("The element Pinit(%d) is < 0.\n",i);
		flag=%f;
		ok=%f;
	end
end
if flag,
	if abs(sum(Pinit)-1)>10e-10,	// check if elements of Pinit sum to 1 ( check only if all Pinit(i)>0 )
		printf("Elements of Pinit must sum to 1.\n");
		ok=%f;
	end
end

// Check of A

if (cPinit~=N)|(cPinit~=cA),	// comparison of the size of A with the length of Pinit
	printf("Sizes of Pinit and A do not match\n"); 
	ok=%f; 
end
check=abs(A*ones(N,1)-1);
for i=1:N
	flag=%t;
	for j=1:N
		if A(i,j)<0,	// check if all elements of A are > 0
			printf("The element A(%d,%d) is < 0\n",i,j);
			flag=%f;
			ok=%f;
		end
	end
	if flag,
		if check(i)>10e-10,	// check if rows of A sum to 1 ( check only if all A(i,j)>0 )
			printf("Row %d of A does not sum to 1\n",i);
			ok=%f;
		end
	end
end

// check of sizes of Seg and Me
for i=1:N
	for m=1:M
		if size(Me(m),1)~=K|size(Me(m),2)~=N 	// check of sizes of Me
			printf("Vector Me('+string(m)+')('+string(i)+') has wrong size\n");
		end
		if size(Seg(m)(i),1)~=K|size(Seg(m)(i),2)~=K	// check of size of Seg
			printf("Matrix Seg('+string(m)+')('+string(i)+') has wrong size\n");
		end
	end
end

// check if submatrix of Seg is symmetric
for i=1:N
	for m=1:M
		flag=%t;
		Segi=Seg(m)(i);
		[rSeg,cSeg]=size(Segi);
		for c=1:K		
			for r=c+1:K
				if abs(Segi(r,c)-Segi(c,r))>%eps,
					printf("The submatrix of Seg for i=%d,m=%d is not symmetric\n",i,m);
					ok=%f;
					flag=%f;
				end
			end
		end
		if flag,
			eigval=spec(Segi);
			Det=det(Segi);
			if or(real(eigval)<0),	// check if submatrix is positiv definit ( only if matrix is symmetric )
				printf("The submatrix of Seg for i=%d,m=%d is not positiv definit\n",i,m);
				ok=%f;
			end
			if Det==0,
				printf("The submatrix of Seg for i=%d,m=%d is not regular\n",i,m);
			end
		end
	end
end

// Check of C
if (size(C,1)~=1 & size(C,2)~=1) then // case of single gaussian
	if (rC~=N)|(cC~=M),	// ckeck of size of C
		printf("Matrix C has wrong size\n"); 
		ok=%f; 
	end
check=abs(C*ones(M,1)-1);
for i=1:N
	flag=%t;
	for j=1:M
		if C(i,j)<0,						// check if all C(i,j)>0
			printf("The element C(%d,%d) is < 0\n",i,j);
			flag=%f;
			ok=%f;
		end
	end
	if flag,	// check if rows of C sum to 1 ( only if all C(i,j)>0 )
		if check(i)>10e-10,
			printf("Row %d of C does not sum to 1\n",i);
			// ok=%f;		// to avoid program termination
						// since in bmgauss B is set to 1e-300
						// an small error is introduced
		end
	end
end
end
// Check of Observation sequence O

	[rO,cO]=size(O);
	if (rO~=size(Seg(1)(1)))
		printf("Wrong dimension of observation matrix O\n");
		ok=%f;
	end





