function [distsym,distnsym]=chmmdist(HMMA,HMMB,O)

// This function calculates the non-symmetric and symetric distances between two HMMs
// The models may have different N or M, however K must be the same
// and are given as a list HMMx (can be generated with the function 'chmmlist'

// HMMA is defined by:

// Pinita = HMMA(1) 	: Initial state distribution vector (1*N) of HMMA
// Aa     = HMMA(2)    	: State stochastic matrix (N*N) of HMMi of HMMA
// Sega    = HMMA(4)	: Observation Covariance list Sega=list(1:M)Segi of HMMA
// Segsi		: Covariance Matrix (K*K) in state si of HMMA
// Mea     = HMMA(3) 	: Observation mean list Me=list(1:M)Mi of HMMA
// Mi			: Matrices  Mi (K*N)of mean vectors Msi of HMMA
// Msi			: Observation Mean vectors (K*1) in state si of HMMA
// Ca     = HMMA(5)     : Mixture gains (for a mixture of Gaussians probability density function) of HMMA
// C=ones(N*1) 		: Case of Single Gaussian
// C (N*M)		: Case of M mixtures
// dist			: distance between HMMA and HMMB (dist=(LPa-LPb)/T);
// LPa	  = HMMA(6)	: log likelihood of O per model i (Lpa=log(P(OIHMMA))
// M			: Number of Gaussian observation mixtures (scalar)
// N			: Number of states in HMM (scalar)
// O     		: Given observation sequence matrix (K*T)
// K			: Observation mean vector dimension
// T     		: Observation sequence length (scalar)
// Pinitb, Ab, Segb, Meb, Cb and LPb are defined by HMMB in the same manner
// as in HMMA
// distnsym		: distance non-symetric between HMMA and HMMB (distnsym=(LPa-LPb)/T);
// distsym		: distance symetric between HMMA and HMMB (distsym=[dist(A,B)+dist(B,A)]/2T);

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=3
	printf("Wrong number of input arguments in function CHMMDIST !\nFunction call is :\n");
	printf("[ dist ] = chmmdist ( HMMi , HMMj , O )");
	printf("Program termination\n");
	abort;
end
if lhs~=1
	printf("Wrong number of output variables.\nContinue? (y/n)\n");
	ans=read(%io(1),1,1,'(a)');
	if ~((ans=='y')|(ans=='Y'))
		printf("Program termination\n");
		abort;
	end
end



[Pinita,Aa,Mea,Sega,Ca,LPa,Coma]=chmmdeli(HMMA);
[Pinitb,Ab,Meb,Segb,Cb,LPb,Comb]=chmmdeli(HMMB);

// check of the consistency of the model parameters
if ~cchkparm(Pinita,Aa,Mea,Sega,Ca,O), 
	printf("Program termination\n");
	abort; 
end	
if ~cchkparm(Piniti,Ai,Mei,Segi,Ci,O), 
	printf("Program termination\n");
	abort; 
end
if ~cchkparm(Pinitb,Ab,Meb,Segb,Cb,O), 
	printf("Program termination\n");
	abort; 
end
[K,T]=size(O);			// K : number of observations per state T : observation length
[Na,Na]=size(Aa);		// Na : number of states in model A
[rMea,Ma]=size(Mea);		// Ma : number of mixtures in model A
[Nb,Nb]=size(Ab);		// Nb : number of states in model B
[rMeb,Mb]=size(Meb);		// Mb : number of mixtures in model B

[LPa,Pa]=clikelih(Pinita,Aa,Mea,Sega,Ca,O);
[LPb,Pb]=clikelih(Pinitb,Ab,Meb,Segb,Cb,O);
distnsym=(LPa-LPb)/T;
distsym=[(LPa-LPb)+(LPb-LPa)]/(2*T);


