function [LP,Pinit,A,Me,Seg,C]=clrreest(Pinit,A,Me,Seg,C,Obs,tol,IterMax)

// Left-to-right CHMM
// Reestimation of the parameters of a HMM until a certain stop criterion is reached.
// The following function calls the fuction "clreesg" or the "clreemg" until the improvement of the normalized log-probability
// i.e. stop if (abs((LPnew-LPold)/LPold))) is less than a given tolerance tol or Iteration > IterMax.
// This function reestimates the CHMM model for many iterations on "cmsrecur". The conditonal probability of
// a (K) dimonsional observation is considered to be a single (M=1) or a
// mixture (M>1) of gaussian densities with mean vectors (Me) and covariance matrices
// (Seg).
// This function takes the underflow problem into account.
// The reestimation formulas are based on Liporace , Juang and Rabiner Algorithms.
// L.Liporace: Maximum likelihood estimation for multivariate observations of Markov sources.
// IEEE. Trans. Inform. Theory, IT-28, pp. 729-734, 1982.
// B. H. Juang. Maximum Likelihood Estimation for Multivariate Mixture Observations of
// Markov Chains. IEEE Transactions on Information Theory, Vol IT-32, No. 2, March 1986.
// L.R.Rabiner et al.: A model-based connected-digit recognition system
// using either HMMs or templates, IEEE Computer Speech and Language 1986
// L.R. Rabiner: A Tutorial on HMM and selected applications in speech recognition, IEEE 1989

// Pinit    : Initial state distribution vector (1*N)
// A        : State stochastic matrix (N*N)
// Seg      : Observation Covariance list Seg=list(1:M)Segi
// Segi     : Observation covarianc list Segi=list(1:N)Segsi
// Segsi    : Covariance Matrix (K*K) in state si
// Me       : Observation mean list Me=list(1:M)Mi
// Mi       : Matrices  Mi (K*N)of mean vectors Msi
// Msi      : Observation Mean vectors (K*1) in state si
// C        : Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*M)  : Case of Single Gaussian
// C (N*M)  : Case of M mixtures
// M        : Number of Gaussian observation mixtures (scalar)
// K        : Observation mean vector dimension
// N        : Number of states in HMM (scalar)
// T        : Observation sequence length (scalar)
// Obs      : The observation list obs consists of the different observation
//            matrices (sequences) of dimension (KxT), where K is the
//            dimension of the observation vectot at time t and T is
//            the sequence length.
// Ts       : Observation sequence length
// O        : Given observation sequence matrix (K*Ts)
// LP       : Normalized Likelihood (log(P(O))/Ts)

// Copyright (c) 2000 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=8,
    printf("Wrong number of input arguments in function CLRREEST. Function call is:\n");
    printf("[ LP , Pinit , A , Seg , Me , C ] = clrreest( Pinit , A , Me , Seg , C , Obs ,  tol , IterMax)\n");
    printf("Program termination\n");
    abort;
end

// check of the consistency of the model parameters
if (size(C,2)==1) then
if lhs~=6,
    printf("Wrong number of output arguments in function CLRREEST. Function call is:\n");
    printf("[ LP , Pinit , A , Seg , Me , C ] = clrreest ( Pinit , A , Me , Seg , C , Obs , tol , IterMax)\n");
    printf("Program termination\n");,
    abort;
end

    if ~cchkparm(Pinit,A,Me,Seg,C,Obs(1)), // check if the sizes of A, Pinit, Me, Seg are consitent
        printf("Program termination\n");
        abort;
    end

    [LP,Pinit,A,Me,Seg]=clrreesg(Pinit,A,Me,Seg,Obs,tol,IterMax);
else
if lhs~=6,
    printf("Wrong number of output arguments in function CLRREESF. Function call is:\n",rhs);
    printf("[ LP , Pinit , A , Seg , Me , C ] = clrreesg ( Pinit , A , Me , Seg , C , Obs , tol , IterMax)\n");
    printf("Program termination\n");
    abort;
end

    if ~cchkparm(Pinit,A,Me,Seg,C,Obs(1)), // check if the sizes of A,Pinit, Me, Seg are consitent
        printf("Program termination\n");
        abort;
    end
    [LP,Pinit,A,Me,Seg,C]=clrreemg(Pinit,A,Me,Seg,C,Obs,tol,IterMax);
end
