function [LP,Pinit,A,Me,Seg,C]=crecurmg(Pinit,A,Me,Seg,C,O)

// Last change 01.03.98   
// This function calculates the Forward and Backward Probabilities 
// and then reestimates the CHMM model for a single iteration. 
// The conditonal probability of a (K) dimonsional observation is 
// considered to be a single or a mixture of m gaussian densities
//  with mean vectors (Mi) and covariance matrices
// (Segi).
// This function takes the underflow problem into account. 
// The reestimation formulas are based on Liporace and Rabiner Algorithms.  crecurmg
// L.Liporace: Maximum likelihood estimation for multivariate observations of Markov sources.
// IEEE. Trans. Inform. Theory, IT-28, pp. 729-734, 1982.  
// L.R.Rabiner et al.: A model-based connected-digit recognition system
// using either HMMs or templates, IEEE Computer Speech and Language 1986 
// L.R. Rabiner: A Tutorial on HMM and selected applications in speech recognition, IEEE 1989

// O     	: Given observation sequence matrix (K*T)
// Pinit 	: Initial state distribution vector (1*N)
// A     	: State stochastic matrix (N*N)
// Seg   	: Observation Covariance list Seg=list(1:M)Segi
// Segi 	: Observation covarianc list Segi=list(1:N)Segsi
// Segsi	: Covariance Matrix (K*K) in state si 
// Me    	: Observation mean list Me=list(1:M)Mi
// Mi		: Matrices  Mi (K*N)of mean vectors Msi
// Msi		: Observation Mean vectors (K*1) in state si 
// C     	: Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*M) 	: Case of Single Gaussian
// C (N*M)	: Case of M mixtures
// M		: Number of Gaussian observation mixtures (scalar)
// K		: Observation mean vector dimension
// N		: Number of states in HMM (scalar)
// T     	: Observation sequence length (scalar)
// LP		: Normalized Likelihood (log(P(O|model))/T
// i 		: state index
// m 		: mixture index

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved


c=(2*%pi)^(-K/2);
[K,T]=size(O);		// K : number of observations per state T : observation length
[N,cA]=size(A);		// N : number of states
Alpha=zeros(N,T);	// Alpha : array to store the forward prob.
Beta=zeros(N,T);	// Beta :array to store the backward prob.
Ro=list();              // Ro : list to store the scaled forward prob for a certain m
M=size(Me);           	// M : number of mixtures
s=zeros(1,T);		// s : scaling factor

// GaussMatm 	: A list of dimension t of matrices (N,M),generated by gengauss.sci
//		  calculates for t=1,..,T  bim(Ot) in state i and mixture m
// GaussMat 	: A list of dimension t of vectors (N,1), the vectors components are bj(Ot)

[GaussMatm,GaussMat]=gengausm(C,Me,Seg,O);

for m=1:M			
	Ro(m)=zeros(N,T);
end
			
// calculation of Alpha and Ro at t=1 and for i=1,..,N 
//t=1;
Alpha(:,1)=Pinit'.*GaussMat(:,1);
if and(Alpha(:,1)==0),
	printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n");
	abort;
end

s(1)=1/sum(Alpha(:,1));
Alpha(:,1)=s(1)*Alpha(:,1);

for m=1:M
	Ro(m)(:,1)=s(1)*Pinit'.*GaussMatm(m)(:,1);
end
// calculation of Alpha and R for t=1,..,T and for i=1,..,N 
for t=2:T
	Alpha(:,t)=(A'*Alpha(:,t-1)).*GaussMat(:,t);
	s(t)=1/sum(Alpha(:,t));
	Alpha(:,t)=s(t)*Alpha(:,t);
	
	if and(Alpha(:,t)==0),
	printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n");
		abort;
	end
	for m=1:M
		Ro(m)(:,t)=s(t)*(A'*Alpha(:,t-1)).*GaussMatm(m)(:,t);
	end
end
// Calculation of P(O\Lambda)		// Probability that the "old" model has generated 
					// the observation sequence
		
//	LP=-sum(log(s));
// Calculation of P(O\Lambda)
if or(s==0) then
  LP= -%inf
else
LP=-sum(log(s))/T; // normalized likelihood		
end
// Calculation of Beta at t=T for i=1,..,N
Beta(:,T)=s(T)*ones(N,1);	
// calculation of Beta for t=T-1,..,1 and for i=1,..,N 

for k=1:T-1
	t=T-k;
	Beta(:,t)=s(t)*A*(GaussMat(:,t+1).*Beta(:,t+1));	
end

// Reestimation of model parameters

// Reestimation of the initial states distribution (Pinit).

AB=Alpha(:,1)'.*Beta(:,1)';

Pinit = AB/sum(AB);                            

// Reestimation of the State transition matrix (A)
// Calculation of the Denomiators for the reestimation

Is=ones(1,T)./s;
SumDC=(Alpha.*Beta)*Is';
SumDA=SumDC-Alpha(:,T).*Beta(:,T)/s(T);

if min(abs(SumDC))==0,
	printf("Warning : Too small values in the denominator in the reestimation of C).\nThis leads to a division by zero.\n");
	printf("Program termination\n");
	abort;	
end

if min(abs(SumDA))==0,
	printf("Warning : Too small values in the denominator in the reestimation of A).\nThis leads to a division by zero.\n");
	printf("Program termination\n");
	abort;	
end

for j=1:N
	A(:,j)=A(:,j).*(Alpha(:,1:T-1)*(GaussMat(j,2:T).*Beta(j,2:T))')./SumDA
end

Sumrobeta=zeros(N,M);
for m=1:M
	Sumrobeta(:,m)=(Ro(m).*Beta)*Is';
end

if min(abs(Sumrobeta))==0,
	printf("Warning : Too small values in the denominator in the reestimation of Me and Seg ).\nThis leads to a division by zero.\n");
	printf("Program termination\n");
	abort;	
end

for i=1:N
	for m=1:M
		RB=(Ro(m)(i,:).*Beta(i,:).*Is)/Sumrobeta(i,m);
		Me(m)(:,i)=O*RB';
		Mi=(O-Me(m)(:,i)*ones(1,T));
		Segi=((ones(K,1)*RB).*Mi)*Mi';
// The following modification prevents the reestimation recursion
// from aborting due to a covariance matrix which is only positive 
// semi-definite by adding to the diagonal a value which is 60dB down
// from the average eigen-value of the matrix. C. Bunks 6-18-97
//                if length(find(spec(Segi)<1e-6*trace(Segi)/K))>0 then
//                   Segi = Segi+1e-6*trace(Segi)/K*eye(K,K);
//                end
// End of modification, C. Bunks 6-18-97              
               Seg(m)(i)=Segi+(%eps^2)*eye(K,K);
	end
end

// Reestimation of gain matrix (C)

for m=1:M
	C(:,m)=Sumrobeta(:,m)./SumDC;
end

	
			












