function [LP,A,B]=danneal(a,b,N,M,O,Iter)

// DHMM training by simulated annealing approach

// This function is used for the estimation of a DHMM 
// by Simulated Annealing, given an observation sequence.
// To be able to treat left to right models, the algorithm uses the 
// penalization matrices a,b to indicate the forbidden transitions and 
// observations. 'a' is the matrix of allowed transitions. 
// They are indicated by '1' and forbidden transitions by '0'. 
// The same is valid for the observation matrix b.
// The variable Iter gives the number of iterations with different ( reducing )
// temperatures
// Brief description of the algorithm : Given the observation sequence O, a 
// state sequence is generated at random and its probability of generating O is 
// calculated. This is called the initial cost. Afterward one state in the 
// sequence is changed at random and the probability that this new state 
// sequence has generated O is calculated. If the latter is bigger than the
// old one, or the probability of acceptance Paccept=exp((Pnew-Pold)/Temp) is 
// big enough, the change is saved. The probability of acceptance decreases 
// with decreasing temperature so that for longer iteration only better 
// solutions are accepted.  

// a :(N*N) Matrix that indicate the allowed transitions (1 for allowed, 0 for forbidden transitions),
// necessary in left-to right models.
// b 		:(N*M) Matrix that indicate the allowed observation in state i (1 for 
//		allowed, 0 for forbidden observation in state i) 
// LP 	 : Normalized log of likelihood of HMM (log(P(O|model)/T)
// P	 : Likelihood
// A     : State stochastic matrix (N*N)
// B     : Observation stochastic matrix (N*M)
// Pinit : Initial state distribution vecor (1*N), Pinit=[1 0 0 ...] 	
// N	 : Number of states	
// M	 : Number of distinct observation symbols per state
// O	 : Observation sequence matrix (1*T)
// Iter		: Number of iterations with different ( reducing ) temperatures

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved


//check of the function call

[lhs,rhs]=argn(0);
if rhs~=6
    printf('Wrong number of input arguments in function DANNEAL.Function call is:\n ');
    printf('[ LP, A , B ] = danneal ( a , b , N , M , O , Iter )');
    printf('Program termination\n');
    abort;
end
if lhs~=3
    printf('Wrong number of output variables\n');
    printf('Continue? (y/n)\n');
    ans=read(%io(1),1,1,'(a)');
    if ~((ans=='y')|(ans=='Y')),
        printf('Program termination\n');
        abort;
    end
end
ExecTime=timer();  
printf('>> Simulated Annealing running for %d Temperature levels\n',Iter);
printf('   Please be patient we will keep you informed of the remaining Time\n')
// Definition of variables

[lO,T]=size(O);        // T : length of observation sequence
Aest=ones(N,N)*%eps;       // A : estimated state stochastic matrix
Best=ones(N,M)*%eps;       // B : estimated observation stochastic matrix
X=zeros(T,1);          // X : guessed state sequence
Temp=0.15*T;           // initial temperature
Chain=T;               // iteration length with fixed temperature
Alpha=0.9;             // reduction factor for the temperature
Beta=1.1;              // increasing factor for the iteration length
rand('uniform');       // sets random number generator to uniform

// Initialization

X(1)=1;                // Pinit is always assumed to be [1 0 0 ....]
Best(X(1),O(1))=Best(X(1),O(1))+1;
for t=2:T
    stop=%f;
    while ~stop,
        X(t)=fix(rand(1)*N)+1;        // generate state at random
        stop=%t;
    end
    Aest(X(t-1),X(t))= Aest(X(t-1),X(t))+1;
    Best(X(t),O(t))=Best(X(t),O(t))+1;
end

// Calculate initial cost : Pest

SumA=sum(Aest,2);
SumB=sum(Best,2);

// probability that the guessed state sequence has generated the observation

Pest=-sum(SumA.*log(SumA))-sum(SumB.*log(SumB))+sum(Aest.*log(Aest).*a)+sum(Best.*log(Best).*b);

BestP=Pest;                // save the best estimated variables so far
BestA=Aest;
BestB=Best;
continue = %t;
ExecTime=timer();
// Start simulated annealing 

for k=1:Iter
    printf('%d iteration. Be patient !\n',k);
    Chain=fix(Chain*Beta);             // Increasing the iteration length
    Temp=Alpha*Temp;                   // Reduction of the temperature
    Accepted=0;
    for trial=1:Chain
        t=fix(rand(1)*(T-1))+2;        // Choice of instant for changing X
        stop=%f;
        while ~stop,                   // Choice of state to change to
            Xnew=fix(rand(1)*N)+1;
            if Xnew ~= X(t), 
                stop=%t;
            end
        end
        Amod=Aest;                     // save old Aest
        Bmod=Best;                     // save old Best
        Amod(X(t-1),X(t))=Amod(X(t-1),X(t))-1;   // change of A according 
                                                 // to the new state sequence
        Amod(X(t-1),Xnew)=Amod(X(t-1),Xnew)+1;
        Bmod(X(t),O(t))=Bmod(X(t),O(t))-1;       // change of B according 
                                                 // to the new state sequence
        Bmod(Xnew,O(t))=Bmod(Xnew,O(t))+1;
        if t < T,
            Amod(X(t),X(t+1))=Amod(X(t),X(t+1))-1;
            Amod(Xnew,X(t+1))=Amod(Xnew,X(t+1))+1;
        end
        SumA=sum(Amod,2);
        SumB=sum(Bmod,2);
        // calculation that the new state sequence has generated the observation
        if ((or(Amod==0))|(or(Bmod==0))|(or(SumA==0))|(or(SumB==0)))then
        	if or(Amod==0)then
        		Amod=Amod+%eps^2*ones(Amod);
        	elseif or(Bmod==0) then
        		Bmod=Bmod+%eps^2*ones(Bmod);
         	elseif or(SumA==0) then
        		SumA=SumA+%eps^2*ones(SumA);
         	elseif or(SumB==0); 
        		SumB=SumB+%eps^2*ones(SumB);
        	end,        		       	
        end,
 Pmod=-sum(SumA.*log(SumA))-sum(SumB.*log(SumB))+sum(Amod.*log(Amod).*a)+sum(Bmod.*log(Bmod).*b);


       // acceptance of the new state sequence if the new probability is 
        // greater then the old one or the probability of acceptance 
        // Paccept=exp((Pmod-Pest)/Temp) is big enough
        
        if ( Pmod > Pest | exp((Pmod-Pest)/Temp) > rand(1)),
	    Accepted = Accepted + 1;
            Pest=Pmod;    // change the estimated variables to the new values
            Aest=Amod;
            Best=Bmod;
            X(t)=Xnew;
            if Pest > BestP, 
                BestA=Aest;    // change the best estimated variables so far
                BestB=Best; 
                BestP=Pest; 
            end
        end
    end
    Tk_1= timer();
    nrem = Iter-k;
    Trem = Tk_1*(Beta*(Beta^nrem - 1)/(Beta - 1));
      select %t
       case (Trem < 300)  then
        printf('** Execution time at Temperature level %d is  %d s. Estimated remaining Time : %d s\n',...
               k,int(Tk_1),int(Trem));
       case (Trem >= 300 & Trem < 3600)  then
        printf('** Execution time at Temperature level %d is  %d s. Estimated remaining Time : %d mn\n',...
               k,int(Tk_1),int(Trem/60));
       else 
        printf('** Execution time at Temperature level %d is  %d s. Estimated remaining Time : %d h %d mn\n',...
               k,int(Tk_1),int(Trem/3600),int((Trem-int(Trem/3600)*3600)/60));
      end  // end of select
     Accepted = Accepted * 100 / Chain;  
     printf('>> Acceptance Rate = %6.2f  Current Normalized Log of Probability = %e\n',Accepted,BestP/T);
end

// calculation of the matrices A and B and further optimisation using 
// the Viterbi algorithm

BestA=inv(diag(sum(BestA,2)))*BestA;       // calculation of A
BestB=inv(diag(sum(BestB,2)))*BestB;       // calculation of B
A=BestA;
B=BestB;
Pinit=zeros(1,N);
Pinit(1)=1;                        // Pinit is always assumed to be [1 0 0 ...]
Prob=BestP;
// optimisation using the Viterbi-algorithm
// The variable Vit gives the number of iterations of an optimisation using the
// Viterbi algorithm following the simulated annealing procedure.
Vit=2;
tol=0.000001;
printf('>> Five iteration for further optimization using the Viterbi algorithm\n');    
[LP,A,B]=dvireest(Pinit,A,B,O,tol,Vit);













