function [LP,Pinit,A,B]=dlrrecur(Pinit,A,B,Obs)

// Left-to-right DHMM
// This function calculates the Forward and Backward Probabilities 
// and then reestimates the DHMM model for multiple obsevations. 
// This is only necessary for left to right models since only these models need
// a training with different observation sequences. 
// ( Pinit(1)=1, Pinit(i)=0, i~=1 )
// ( A(i,j)=0,j>i+delta ).
// If there are problems with divisions by zero which might be caused by a bad 
// initial guess of the model parameters, a better initial guess often solves 
// this.
// The reestimation formulas are obtained by Liporace Algorithm.  
// This function takes the underflow problem into account. 
// LP : log of mean probability that the new model has generated the given 
// observation sequences.
// References :
// * L.R. Rabiner et al. : A model-based connected-digit recognition 
// system using either hidden Markov Models or templates, IEEE Computer 
// Speech and  Language (1986) 1, pp. 167-197. 
// * L.R. Rabiner : A Tutorial on HMM and selected applications in speech 
// recognition, IEEE 1989.

// A     : State stochastic matrix (N*N)
// B     : Observation stochastic matrix (N*M)
// Pinit : Initial state distribution vecor (1*N)
// Obs	 : Observation sequence vector (1*T)	
// 	   The observation matrix O (row vector) consists of the different observation
// 	   matrices which are preceded by a column containing their length 
// 	   e.g. three different observation matrices of length 
//         100 120 80 give the matrix 
//         Obs=[column with the first length (100),columns with the belonging 
//         observation, column with the second length (120), columns with the 
//         belonging observation etc.]
// M	 : Number of distinct observation symbols per state
// N	 : Number of states	
// T	 : Observation sequence length
// LP 	 : Mean of the normalized log of likelihood of HMM (log(P(O|model)/T)
// P	 : Likelihood

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=4,
    printf('Wrong number of input arguments in function DMSRECUR.Function call is:\n');
    printf('[ LP , Pinit , A , B ] = dmsrecur ( Pinit , A , B , Obs )\n');
    printf('Program termination\n');
    abort;
end
if lhs~=4,
    printf('Wrong number of output arguments. Continue ? ( y/n )\n');
    t=read(%io(1),1,1,'(a)');
    if (t~='y')&(t~='Y'),
        printf('Program termination\n');
        abort;
    end
end

// check of the consistency of the model parameters

if ~dchkpar(Pinit,A,B),       
    printf('Program termination\n');
    abort;
end

[rObs,L]=size(Obs);     // L : length of all observations sequences + number of 
                     	//     different observations sequences
[N,N]=size(A);       	// N : number of states
[N,M]=size(B);       	// M : number of possible observations

// check of Pinit

if Pinit(1)~=1,
    printf('It has to be Pinit(0)=1 for left to right models\n'); 
    printf('Program termination\n');
    abort;
end
for i=2:N
    if Pinit(i)~=0,
        printf('It has to be Pinit(i)=0, i>1 for left to right models\n'); 
        printf('Program termination\n');
        abort;
    end
end

SumDAl=zeros(N,1);    // Denominator to reestimate A for a single observation
DA=SumDAl;            // Denominator to reestimate A for the set of 
                      // observations
SumDBl=SumDAl         // Denominator to reestimate B for a single observation
DB=DA;                // Denominator to reestimate B for the set of 
                      // observations
NAl=zeros(N,N);       // Nominator to reestimate A for a single observation
NA=NAl;               // Nominator to reestimate A for the set of observations
NBl=zeros(N,M);       // Nominator to reestimate B for a single observation
NB=NBl;               // Nominator to reestimate B for the set of observations
LP=0;

l=1; 
while l<L

    	T=Obs(l);
    	O=Obs(l+1:l+T);        // one observation sequence taken from the 
                           // observation matrix Obs
    	l=l+T+1;
    	printf('Iteration with the observation of length T=%d\n',T);
    	Alpha=zeros(N,T);      // Alpha : array to store the forward prob.
    	Beta=zeros(N,T);       // array to store the backward prob.
    	s=zeros(1,T);
// calculation of Alpha at t=1 and for i=1,..,N 
    	t=1;
	Alpha(:,1)=Pinit'.*B(:,O(1,1));
	s(1)=1/sum(Alpha(:,1));
	Alpha(:,1)=s(1)*Alpha(:,1);                    

	if and(Alpha(:,1)==0),
		printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n",t);
	abort;
	end

// calculation of Alpha for t=1,..,T and for i=1,..,N 

	for t=2:T
    		Alpha(:,t)=(A'*Alpha(:,t-1)).*B(:,O(1,t));                          
    		s(t)=1/sum(Alpha(:,t));
    		Alpha(:,t)=s(t)*Alpha(:,t);                
    		if and(Alpha(:,t)==0),
			printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n",t);
			abort;
    		end
	end
                               
   	if or(s==0) then
      	LP= -%inf
   	else
     	LPl=-sum(log(s))/T; // normalized log of likelihood	
	LP=LP+LPl;	
   	end

// Calculation of Beta at t=T for i=1,..,N
	Beta(:,T)=s(T)*ones(N,1);	
// calculation of Beta for t=T-1,..,1 and for i=1,..,N 

    	for k=1:T-1
		t=T-k;
		Beta(:,t)=s(t)*A*(B(:,O(1,t+1)).*Beta(:,t+1));	
    	end
// Reestimation of model parameters

// the scales variables Alphas and Betas contain already a division by 
// the probability that the actual observation was generated by the 
// model so that Rabiner (109) is fullfilled automatically

// Calculation of the Denomiators for the reestimation
	Is=ones(1,T)./s;
	SumDBl=(Alpha.*Beta)*Is';
	SumDAl=SumDBl-Alpha(:,T).*Beta(:,T)/s(T);

	if min(abs(SumDBl))==0,
		printf("Warning : Too small values in the denominator in the reestimation of B).\nThis leads to a division by zero.\n");
		printf("Program termination\n");
		abort;	
	end    
    	DB=DB+SumDBl;
    	DA=DA+SumDAl;

// Calculation of the nominator for the reestimation of the state 
// transition matrix (A)
		for j=1:N
  			NAl(:,j)= A(:,j).*(Alpha(:,1:T-1)*(B(j,O(2:T)).*Beta(j,2:T))')  
		end
    	NA=NA+NAl;

// Calculation of the nominator for the reestimation of the observation 
// symbol probability matrix (B)

        for k = 1:M
            SumNBjk=0;
            for t=1:T
                if O(t)==k 
                SumNBjk=SumNBjk+Alpha(:,t).*(Beta(:,t)./s(t));
                end
            end
            NBl(:,k)=SumNBjk;            
        end
    
    NB=NB+NBl;
end

// Re-estimation of the model parameters

    A=NA./(DA*ones(1,N));
    B=NB./(DB*ones(1,M));







