function [LP,Pinit,A,B]=dlrreest(Pinit,A,B,Obs,tol,IterMax)

// Function to reestimate the parameters of a Left-to-Right DHMM with multiple 
// observation sequence until a certain stop criterion is reached.  
// This function calls the function drecurs until the improvement of the absolute value of the relative normalized 
// log-probability (abs(LProb-LProbold)/LProbold) is less than a given tolerance tol
// or the number of iterations exceeds IterMax 

// A     : State stochastic matrix (N*N)
// B     : Observation stochastic matrix (N*M)
// Pinit : Initial state distribution vecor (1*N)
// Obs	 : Observation sequence matrix 	
// 	   The observation matrix Obs (row vector) consists of the different observation
// 	   matrices which are preceeded by a column containing their length 
// 	   e.g. three different observation matrices of length 
//         100 120 80 give the matrix 
//         Obs=[column with the first length (100),columns with the belonging 
//         observation, column with the second length (120), columns with the 
//         belonging observation etc.]	 	
// M	 : Number of distinct observation symbols per state
// N	 : Number of states	
// T	 : Observation sequence length
// LP 	 : Normalized log of likelihood of HMM (log(P(O|model)/T)
// P	 : Likelihood
// tol 	 : Stop if abs((New log(likelihood) - old log(liklihood)) / old log(likelihood)) is less than tol or Iteration > IterMax.
// IterMax : Maximum Iteration number

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

[lhs,rhs]=argn(0);
if rhs~=6,            // check if number of input arguments is correct
    printf("Wrong number of input arguments in function DLRREEST.Function call is: \n");
    printf("[ LP , Pinit , A , B ] = dlrreest( Pinit , A , B , O , tol, IterMax )");
    abort;
end
if lhs~=4,            // check if number of output arguments is correct
    printf("Wrong number of output arguments.\n");
    printf("If you want to continue type  y  else  n : \n");
    t=read(%io(1),1,1,'(a)');
    if (t~='y')&(t~='Y')
        abort;
    end
end

if ~dchkpar(Pinit,A,B)
    abort
end
printf("====> Requested tolerance tol (to be compared to RCNL) = %5.10f\n",tol);
printf("====> Requested Maximum iteration IterMax (if tol is not satisfied) = %d\n",IterMax);
printf("**> Iteration = 1\n");
[LProb,Pinit,A,B]=dlrrecur(Pinit,A,B,Obs);
printf("-> Normalized Log of Likelihood (log(P(O))/T) = %5.10f\n",LProb);
LProbold=LProb;
LP=LProb;
printf("**> Iteration = 2\n");
[LProb,Pinit,A,B]=dlrrecur(Pinit,A,B,Obs);
printf("-> Normalized Log of Likelihood (log(P(O))/T) = %5.10f\n",LProb);
RCNL=abs((LProb-LProbold)/LProbold);
printf(">> Relative change in Normalized Log of Likelihood (RCNL=abs((New log(likelihood) - old log(liklihood)) / old log(likelihood))) = %5.10f\n",RCNL);	
Iter=2;
LP=[LP LProb];
while ((RCNL>tol)&(Iter<IterMax))   
    	LProbold=LProb        
	Iter=Iter+1;
	printf("**> Iteration = %d\n",Iter);
	RCNL_old=RCNL;
    	[LProb,Pinit,A,B]=dlrrecur(Pinit,A,B,Obs);
	printf("-> Normalized Log of Likelihood (log(P(O))/T) = %5.10f\n",LProb);
	RCNL=abs((LProb-LProbold)/LProbold);
	RCNL_new=RCNL;
	printf(">> Relative change in Normalized Log of Likelihood (RCNL=abs((New log(likelihood) - old log(liklihood)) / old log(likelihood))) = %5.10f\n",RCNL);	
   	LP=[LP LProb];
end


